<?php

namespace Drupal\symfony_mailer_lite_microsoft\Transport;

use Drupal\Core\Cache\CacheBackendInterface;
use Psr\EventDispatcher\EventDispatcherInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\Mailer\Envelope;
use Symfony\Component\Mailer\Exception\HttpTransportException;
use Symfony\Component\Mailer\SentMessage;
use Symfony\Component\Mailer\Transport\AbstractApiTransport;
use Symfony\Component\Mime\Address;
use Symfony\Component\Mime\Email;
use Symfony\Contracts\HttpClient\HttpClientInterface;
use Symfony\Contracts\HttpClient\ResponseInterface;

class GraphApiTransport extends AbstractApiTransport {

  private const CACHE_KEY = 'email-token';

  public function __construct(
    private string $graphTentantId,
    private string $graphClientId,
    private string $graphClientSecret,
    private string $from,
    protected CacheBackendInterface $cache,
    ?HttpClientInterface $client = NULL,
    ?EventDispatcherInterface $dispatcher = NULL,
    ?LoggerInterface $logger = NULL,
  ) {
    parent::__construct($client, $dispatcher, $logger);
  }

  public function __toString(): string {
    return sprintf('microsoft-graph-api://%s:{SECRET}@%s', $this->graphClientId, $this->graphTentantId);
  }

  public function getToken(): string {
    return $this->cache->get(self::CACHE_KEY)?->data ?? $this->requestAccessToken();
  }

  protected function doSendApi(SentMessage $sentMessage, Email $email, Envelope $envelope): ResponseInterface {
    $token = $this->getToken();
    $logger = $this->getLogger();

    $response = $this->client->request('POST', $this->getEndpoint($sentMessage), [
      'json' => $this->normalizeEmail($email, $envelope),
      'auth_bearer' => $token,
    ]);

    try {
      $statusCode = $response->getStatusCode();
    }
    catch (\Exception $e) {
      throw new HttpTransportException('Could not reach Microsoft Graph API.', $response, 0, $e);
    }

    if (202 !== $statusCode) {
      $logger->error('Unable to sent e-mail using Graph API: %error.', [
        '%error' => $response->getContent(FALSE),
      ]);
      throw new HttpTransportException('Unable to sent e-mail using Graph API', $response);
    }

    return $response;
  }

  private function normalizeEmail(Email $email, Envelope $envelope): array {

    $payload = [
      'message' => [
        'subject' => $email->getSubject(),
        'toRecipients' => $this->normalizeAddresses($email->getTo()),
        'ccRecipients' => $this->normalizeAddresses($email->getCc()),
        'bccRecipients' => $this->normalizeAddresses($email->getBcc()),
        'body' => $this->normalizeBody($email),
        'attachments' => $this->normalizeAttachments($email),
      ],
      'saveToSentItems' => $this->normalizeSaveToSentItems($email),
    ];

    return $payload;
  }

  private function normalizeAddress(Address $address): array {
    $addressArray = [
      'emailAddress' => [
        'address' => $address->getAddress(),
      ],
    ];
    if ($address->getName()) {
      $addressArray['emailAddress']['name'] = $address->getName();
    }

    return $addressArray;
  }

  /**
   * Normalize an array of addresses to the required format.
   *
   * @param \Symfony\Component\Mime\Address[] $addresses
   *   An array of address objects.
   *
   * @return array
   *   An array of normalized addresses.
   */
  private function normalizeAddresses(array $addresses): array {
    $addressesArray = [];
    foreach ($addresses as $address) {
      $addressesArray[] = $this->normalizeAddress($address);
    }

    return $addressesArray;
  }

  private function normalizeBody(Email $email): array {
    // Prefer html body
    if (NULL !== $htmlContent = $email->getHtmlBody()) {
      return [
        'contentType' => 'html',
        'content' => $htmlContent,
      ];
    }

    // Fallback on textBody
    if (NULL !== $textContent = $email->getTextBody()) {
      return [
        'contentType' => 'text',
        'content' => $textContent,
      ];
    }

    return [];
  }

  private function normalizeAttachments(Email $email): array {
    $attachments = [];
    foreach ($email->getAttachments() as $attachment) {
      $headers = $attachment->getPreparedHeaders();
      $filename = $headers->getHeaderParameter('Content-Disposition', 'filename');

      $attachments[] = [
        '@odata.type' => '#microsoft.graph.fileAttachment',
        'contentType' => $headers->get('Content-Type')->getBody(),
        'contentBytes' => base64_encode($attachment->getBody()),
        'name' => $filename,
      ];
    }

    return $attachments;
  }

  private function normalizeSaveToSentItems($email): bool {
    $saveToSentItems = TRUE;
    $saveToSentHeader = $email->getHeaders()->get('X-Save-To-Sent-Items');
    if ($saveToSentHeader !== NULL) {
      if (strtolower($saveToSentHeader->getBodyAsString()) === 'false') {
        $saveToSentItems = FALSE;
      }
    }

    return $saveToSentItems;
  }

  private function requestAccessToken(): string {
    $url = 'https://login.microsoftonline.com/' . $this->graphTentantId . '/oauth2/v2.0/token';

    $response = $this->client->request('POST', $url, [
      'body' => [
        'client_id' => $this->graphClientId,
        'client_secret' => $this->graphClientSecret,
        'scope' => 'https://graph.microsoft.com/.default',
        'grant_type' => 'client_credentials',
      ],
    ]);

    $token = json_decode($response->getContent(), NULL, 512, JSON_THROW_ON_ERROR);
    $access_token = $token->access_token;
    $expiresIn = $token->expires_in ?? time() + 300;

    // Subtracting 60 seconds from the TTL
    // as a safety margin to certainly not use an expiring token.
    $expiryTime = $expiresIn - 60;
    $this->cache->set(self::CACHE_KEY, $access_token, $expiryTime);
    return $access_token;
  }

  private function getEndpoint(SentMessage $sentMessage): string {
    $senderAddress = $sentMessage->getEnvelope()->getSender()->getAddress();

    // Use the provided "from" address if available.
    $from = $this->from ?? $senderAddress;

    return sprintf('https://graph.microsoft.com/v1.0/users/%s/sendMail', $from);
  }

}
