<?php

namespace Drupal\Tests\symfony_mailer_lite_microsoft\Unit;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\symfony_mailer_lite_microsoft\Plugin\SymfonyMailerLite\Transport\MicrosoftTransport;
use Drupal\Tests\UnitTestCase;
use Symfony\Component\Mailer\Envelope;
use Symfony\Component\Mime\Address;
use Symfony\Component\Mime\Email;
use Symfony\Contracts\HttpClient\HttpClientInterface;
use Symfony\Contracts\HttpClient\ResponseInterface;

/**
 * @coversDefaultClass \Drupal\symfony_mailer_lite_microsoft\Plugin\SymfonyMailerLite\Transport\MicrosoftTransport
 * @group symfony_mailer_lite_microsoft
 */
class MicrosoftTransportTest extends UnitTestCase {

  protected $stringTranslation;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->stringTranslation = $this->createMock(TranslationInterface::class);
    $this->stringTranslation->method('translate')
      ->willReturnCallback(function ($string) {
        return $string;
      });
  }

  /**
   * Creates a Microsoft Transport instance with translation service.
   */
  protected function createMicrosoftTransport(array $configuration = []): MicrosoftTransport {
    $transport = new MicrosoftTransport($configuration, 'microsoft_graph_api', []);
    $transport->setStringTranslation($this->stringTranslation);
    return $transport;
  }

  /**
   * Test configuration and email sending.
   */
  public function testMicrosoftTransport(): void {
    // Test data.
    $config = [
      'user' => 'test@example.com',
      'tenant' => 'test-tenant-id',
      'client_id' => 'test-client-id',
      'client_secret' => 'test-client-secret',
    ];

    // Create mock objects.
    $httpClient = $this->createMock(HttpClientInterface::class);
    $cache = $this->createMock(CacheBackendInterface::class);
    $response = $this->createMock(ResponseInterface::class);

    // Configure response mock.
    $response->method('getStatusCode')
      ->willReturn(202);

    // Configure HTTP client mock for email sending.
    $httpClient->expects($this->once())
      ->method('request')
      ->with(
        'POST',
        $this->anything(),
        $this->callback(function ($options) {
          return isset($options['auth_bearer'])
            && $options['auth_bearer'] === 'test-token-12345'
            && isset($options['json'])
            && isset($options['json']['message']);
        })
      )
      ->willReturn($response);

    // Create test email.
    $email = new Email();
    $email->from(new Address($config['user']))
      ->to(new Address('recipient@example.com'))
      ->subject('Test Subject')
      ->text('Test Content');

    $envelope = new Envelope(
      new Address($config['user']),
      [new Address('recipient@example.com')]
    );

    // Create transport instance using our test class.
    $transport = new TestGraphApiTransport(
      $config['tenant'],
      $config['client_id'],
      $config['client_secret'],
      $config['user'],
      $cache,
      $httpClient
    );

    // Test sending email.
    try {
      $sentMessage = $transport->send($email, $envelope);
      // Verify that a SentMessage object is returned
      $this->assertNotNull($sentMessage);
      $this->assertInstanceOf(\Symfony\Component\Mailer\SentMessage::class, $sentMessage);
    }
    catch (\Exception $e) {
      $this->fail('Email sending failed: ' . $e->getMessage());
    }

    // Test DSN generation.
    $microsoftTransport = $this->createMicrosoftTransport();
    $form_state = new FormState();
    $form_state->setValues($config);

    // Test form submission.
    $form = [];
    $microsoftTransport->submitConfigurationForm($form, $form_state);

    // Verify DSN format.
    $expected_dsn = sprintf(
      'microsoft-graph-api://%s:%s@%s?from=%s',
      $config['client_id'],
      $config['client_secret'],
      $config['tenant'],
      $config['user']
    );

    $this->assertEquals($expected_dsn, $microsoftTransport->getDsn());
  }

  /**
   * Test configuration form building.
   */
  public function testConfigurationForm(): void {
    // Provide default configuration
    $default_config = [
      'user' => '',
      'tenant' => '',
      'client_id' => '',
      'client_secret' => '',
    ];

    // Create transport with default configuration
    $transport = $this->createMicrosoftTransport($default_config);
    $form_state = new FormState();

    $form = $transport->buildConfigurationForm([], $form_state);

    // Verify form structure
    $this->assertArrayHasKey('user', $form);
    $this->assertArrayHasKey('tenant', $form);
    $this->assertArrayHasKey('client_id', $form);
    $this->assertArrayHasKey('client_secret', $form);

    // Verify field types
    $this->assertEquals('textfield', $form['user']['#type']);
    $this->assertEquals('textfield', $form['tenant']['#type']);
    $this->assertEquals('textfield', $form['client_id']['#type']);
    $this->assertEquals('textfield', $form['client_secret']['#type']);

    // Verify each field has a default value set (even if empty)
    $this->assertArrayHasKey('#default_value', $form['user']);
    $this->assertArrayHasKey('#default_value', $form['tenant']);
    $this->assertArrayHasKey('#default_value', $form['client_id']);
    $this->assertArrayHasKey('#default_value', $form['client_secret']);

    // Test form submission with new values
    $test_values = [
      'user' => 'test@example.com',
      'tenant' => 'test-tenant',
      'client_id' => 'test-client-id',
      'client_secret' => 'test-secret',
    ];

    $form_state->setValues($test_values);
    $transport->submitConfigurationForm($form, $form_state);

    // Verify the form values were properly set
    $this->assertEquals($test_values['user'], $form_state->getValue('user'));
    $this->assertEquals($test_values['tenant'], $form_state->getValue('tenant'));
    $this->assertEquals($test_values['client_id'], $form_state->getValue('client_id'));
    $this->assertEquals($test_values['client_secret'], $form_state->getValue('client_secret'));
  }

}
